@@ -6,10 +6,37 @@
 
 #include "fdiskP.h"
 
-/*
- * Alignment according to logical granulity (usually 1MiB)
+/**
+ * SECTION: alignment
+ * @title: Alignment
+ * @short_description: functions to align partitions and work with disk topology and geometry
+ *
+ * The libfdisk aligns the end of the partitions to make it possible to align
+ * the next partition to the "grain" (see fdisk_get_grain_size()). The grain is
+ * usually 1MiB (or more for devices where optimal I/O is greater than 1MiB).
+ *
+ * It means that the library does not align strictly to physical sector size
+ * (or minimal or optimal I/O), but it uses greater granularity. It makes
+ * partition tables more portable. If you copy disk layout from 512-sector to
+ * 4K-sector device, all partitions are still aligned to physical sectors.
+ *
+ * This unified concept also makes partition tables more user friendly, all
+ * tables look same, LBA of the first partition is 2048 sectors everywhere, etc.
+ *
+ * It's recommended to not change any alignment or device properties. All is
+ * initialized by default by fdisk_assign_device().
+ *
+ * Note that terminology used by libfdisk is: 
+ *   - device properties: I/O limits (topology), geometry, sector size, ...
+ *   - alignment: first, last LBA, grain, ...
+ *
+ * The alignment setting may be modified by disk label driver.
  */
-static int lba_is_aligned(struct fdisk_context *cxt, sector_t lba)
+
+/*
+ * Alignment according to logical granularity (usually 1MiB)
+ */
+static int lba_is_aligned(struct fdisk_context *cxt, fdisk_sector_t lba)
 {
 	unsigned long granularity = max(cxt->phy_sector_size, cxt->min_io_size);
 	uintmax_t offset;
@@ -24,7 +51,7 @@ static int lba_is_aligned(struct fdisk_context *cxt, sector_t lba)
 /*
  * Alignment according to physical device topology (usually minimal i/o size)
  */
-static int lba_is_phy_aligned(struct fdisk_context *cxt, sector_t lba)
+static int lba_is_phy_aligned(struct fdisk_context *cxt, fdisk_sector_t lba)
 {
 	unsigned long granularity = max(cxt->phy_sector_size, cxt->min_io_size);
 	uintmax_t offset = (lba * cxt->sector_size) & (granularity - 1);
@@ -32,17 +59,26 @@ static int lba_is_phy_aligned(struct fdisk_context *cxt, sector_t lba)
 	return !((granularity + cxt->alignment_offset - offset) & (granularity - 1));
 }
 
-/*
- * Align @lba in @direction FDISK_ALIGN_{UP,DOWN,NEAREST}
+/**
+ * fdisk_align_lba:
+ * @cxt: context
+ * @lba: address to align
+ * @direction: FDISK_ALIGN_{UP,DOWN,NEAREST}
+ *
+ * This function aligns @lba to the "grain" (see fdisk_get_grain_size()). If the
+ * device uses alignment offset then the result is moved according the offset
+ * to be on the physical boundary.
+ *
+ * Returns: alignment LBA.
  */
-sector_t fdisk_align_lba(struct fdisk_context *cxt, sector_t lba, int direction)
+fdisk_sector_t fdisk_align_lba(struct fdisk_context *cxt, fdisk_sector_t lba, int direction)
 {
-	sector_t res;
+	fdisk_sector_t res;
 
 	if (lba_is_aligned(cxt, lba))
 		res = lba;
 	else {
-		sector_t sects_in_phy = cxt->grain / cxt->sector_size;
+		fdisk_sector_t sects_in_phy = cxt->grain / cxt->sector_size;
 
 		if (lba < cxt->first_lba)
 			res = cxt->first_lba;
@@ -81,32 +117,53 @@ sector_t fdisk_align_lba(struct fdisk_context *cxt, sector_t lba, int direction)
 	return res;
 }
 
-/*
+/**
+ * fdisk_align_lba_in_range:
+ * @cxt: context
+ * @lba: LBA
+ * @start: range start
+ * @stop: range stop
+ *
  * Align @lba, the result has to be between @start and @stop
+ *
+ * Returns: aligned LBA
  */
-sector_t fdisk_align_lba_in_range(struct fdisk_context *cxt,
-				  sector_t lba, sector_t start, sector_t stop)
+fdisk_sector_t fdisk_align_lba_in_range(struct fdisk_context *cxt,
+				  fdisk_sector_t lba, fdisk_sector_t start, fdisk_sector_t stop)
 {
+	fdisk_sector_t res;
+
 	start = fdisk_align_lba(cxt, start, FDISK_ALIGN_UP);
 	stop = fdisk_align_lba(cxt, stop, FDISK_ALIGN_DOWN);
 	lba = fdisk_align_lba(cxt, lba, FDISK_ALIGN_NEAREST);
 
 	if (lba < start)
-		return start;
+		res = start;
 	else if (lba > stop)
-		return stop;
-	return lba;
+		res = stop;
+	else
+		res = lba;
+
+	DBG(CXT, ul_debugobj(cxt, "LBA %ju range:<%ju..%ju>, result: %ju",
+				(uintmax_t) lba,
+				(uintmax_t) start,
+				(uintmax_t) stop,
+				(uintmax_t) res));
+	return res;
 }
 
-/*
- * Print warning if the partition @lba (start of the @partition) is not
- * aligned to physical sector boundary.
+/**
+ * fdisk_lba_is_phy_aligned:
+ * @cxt: context
+ * @lba: LBA to check
+ *
+ * Check if the @lba is aligned to physical sector boundary.
+ *
+ * Returns: 1 if aligned.
  */
-void fdisk_warn_alignment(struct fdisk_context *cxt, sector_t lba, int partition)
+int fdisk_lba_is_phy_aligned(struct fdisk_context *cxt, fdisk_sector_t lba)
 {
-	if (!lba_is_phy_aligned(cxt, lba))
-		fdisk_warnx(cxt, _("Partition %i does not start on physical sector boundary.\n"),
-			partition + 1);
+	return lba_is_phy_aligned(cxt, lba);
 }
 
 static unsigned long get_sector_size(int fd)
@@ -136,8 +193,12 @@ static void recount_geometry(struct fdisk_context *cxt)
  * @heads: user specified heads
  * @sectors: user specified sectors
  *
- * Overrides autodiscovery and apply user specified geometry. The function
- * fdisk_reset_device_properties() restores the original setting.
+ * Overrides auto-discovery. The function fdisk_reset_device_properties()
+ * restores the original setting.
+ *
+ * The difference between fdisk_override_geometry() and fdisk_save_user_geometry()
+ * is that saved user geometry is persistent setting and it's applied always
+ * when device is assigned to the context or device properties are reseted.
  *
  * Returns: 0 on success, < 0 on error.
  */
@@ -168,6 +229,20 @@ int fdisk_override_geometry(struct fdisk_context *cxt,
 	return 0;
 }
 
+/**
+ * fdisk_save_user_geometry:
+ * @cxt: context
+ * @cylinders: C
+ * @heads: H
+ * @sectors: S
+ *
+ * Save user defined geometry to use it for partitioning.
+ *
+ * The user properties are applied by fdisk_assign_device() or
+ * fdisk_reset_device_properties().
+
+ * Returns: <0 on error, 0 on success.
+ */
 int fdisk_save_user_geometry(struct fdisk_context *cxt,
 			    unsigned int cylinders,
 			    unsigned int heads,
@@ -191,6 +266,19 @@ int fdisk_save_user_geometry(struct fdisk_context *cxt,
 	return 0;
 }
 
+/**
+ * fdisk_save_user_sector_size:
+ * @cxt: context
+ * @phy: physical sector size
+ * @log: logicla sector size
+ *
+ * Save user defined sector sizes to use it for partitioning.
+ *
+ * The user properties are applied by fdisk_assign_device() or 
+ * fdisk_reset_device_properties().
+ *
+ * Returns: <0 on error, 0 on success.
+ */
 int fdisk_save_user_sector_size(struct fdisk_context *cxt,
 				unsigned int phy,
 				unsigned int log)
@@ -206,6 +294,12 @@ int fdisk_save_user_sector_size(struct fdisk_context *cxt,
 	return 0;
 }
 
+/**
+ * fdisk_has_user_device_properties:
+ * @cxt: context
+ *
+ * Returns: 1 if user specified any properties
+ */
 int fdisk_has_user_device_properties(struct fdisk_context *cxt)
 {
 	return (cxt->user_pyh_sector
@@ -271,6 +365,20 @@ void fdisk_zeroize_device_properties(struct fdisk_context *cxt)
 	memset(&cxt->geom, 0, sizeof(struct fdisk_geometry));
 }
 
+/**
+ * fdisk_reset_device_properties:
+ * @cxt: context
+ *
+ * Resets and discovery topology (I/O limits), geometry, re-read the first
+ * rector on the device if necessary and apply user device setting (geometry
+ * and sector size), then initialize alignment according to label driver (see
+ * fdisk_reset_alignment()).
+ *
+ * You don't have to use this function by default, fdisk_assign_device() is
+ * smart enough to initialize all necessary setting.
+ *
+ * Returns: 0 on success, <0 on error.
+ */
 int fdisk_reset_device_properties(struct fdisk_context *cxt)
 {
 	int rc;
@@ -297,7 +405,7 @@ int fdisk_reset_device_properties(struct fdisk_context *cxt)
  */
 int fdisk_discover_geometry(struct fdisk_context *cxt)
 {
-	sector_t nsects;
+	fdisk_sector_t nsects;
 
 	assert(cxt);
 	assert(cxt->geom.heads == 0);
@@ -305,9 +413,13 @@ int fdisk_discover_geometry(struct fdisk_context *cxt)
 	DBG(CXT, ul_debugobj(cxt, "%s: discovering geometry...", cxt->dev_path));
 
 	/* get number of 512-byte sectors, and convert it the real sectors */
-	if (!blkdev_get_sectors(cxt->dev_fd, &nsects))
+	if (!blkdev_get_sectors(cxt->dev_fd, (unsigned long long *) &nsects))
 		cxt->total_sectors = (nsects / (cxt->sector_size >> 9));
 
+	DBG(CXT, ul_debugobj(cxt, "total sectors: %ju (ioctl=%ju)",
+				(uintmax_t) cxt->total_sectors,
+				(uintmax_t) nsects));
+
 	/* what the kernel/bios thinks the geometry is */
 	blkdev_get_geometry(cxt->dev_fd, &cxt->geom.heads, (unsigned int *) &cxt->geom.sectors);
 
@@ -391,9 +503,9 @@ static int has_topology(struct fdisk_context *cxt)
  *
  * Returns: 0 on error or number of logical sectors.
  */
-sector_t fdisk_topology_get_first_lba(struct fdisk_context *cxt)
+static fdisk_sector_t topology_get_first_lba(struct fdisk_context *cxt)
 {
-	sector_t x = 0, res;
+	fdisk_sector_t x = 0, res;
 
 	if (!cxt)
 		return 0;
@@ -431,13 +543,7 @@ sector_t fdisk_topology_get_first_lba(struct fdisk_context *cxt)
 	return res;
 }
 
-/*
- * The LBA of the first partition is based on the device geometry and topology.
- * This offset is generic generic (and recommended) for all labels.
- *
- * Returns: 0 on error or number of bytes.
- */
-unsigned long fdisk_topology_get_grain(struct fdisk_context *cxt)
+static unsigned long topology_get_grain(struct fdisk_context *cxt)
 {
 	unsigned long res;
 
@@ -464,7 +570,8 @@ unsigned long fdisk_topology_get_grain(struct fdisk_context *cxt)
  * fdisk_reset_alignment:
  * @cxt: fdisk context
  *
- * Resets alignment setting to the default or label specific values.
+ * Resets alignment setting to the default and label specific values. This
+ * function does not change device properties (I/O limits, geometry etc.).
  *
  * Returns: 0 on success, < 0 in case of error.
  */
@@ -478,8 +585,8 @@ int fdisk_reset_alignment(struct fdisk_context *cxt)
 	DBG(CXT, ul_debugobj(cxt, "reseting alignment..."));
 
 	/* default */
-	cxt->grain = fdisk_topology_get_grain(cxt);
-	cxt->first_lba = fdisk_topology_get_first_lba(cxt);
+	cxt->grain = topology_get_grain(cxt);
+	cxt->first_lba = topology_get_first_lba(cxt);
 	cxt->last_lba = cxt->total_sectors - 1;
 
 	/* overwrite default by label stuff */
@@ -494,18 +601,26 @@ int fdisk_reset_alignment(struct fdisk_context *cxt)
 }
 
 
-sector_t fdisk_scround(struct fdisk_context *cxt, sector_t num)
+fdisk_sector_t fdisk_scround(struct fdisk_context *cxt, fdisk_sector_t num)
 {
-	sector_t un = fdisk_context_get_units_per_sector(cxt);
+	fdisk_sector_t un = fdisk_get_units_per_sector(cxt);
 	return (num + un - 1) / un;
 }
 
-sector_t fdisk_cround(struct fdisk_context *cxt, sector_t num)
+fdisk_sector_t fdisk_cround(struct fdisk_context *cxt, fdisk_sector_t num)
 {
-	return fdisk_context_use_cylinders(cxt) ?
-			(num / fdisk_context_get_units_per_sector(cxt)) + 1 : num;
+	return fdisk_use_cylinders(cxt) ?
+			(num / fdisk_get_units_per_sector(cxt)) + 1 : num;
 }
 
+/**
+ * fdisk_reread_partition_table:
+ * @cxt: context
+ *
+ * Force *kernel* to re-read partition table on block devices.
+ *
+ * Returns: 0 on success, < 0 in case of error.
+ */
 int fdisk_reread_partition_table(struct fdisk_context *cxt)
 {
 	int i;
