@@ -56,6 +56,7 @@
 #include "c.h"
 #include "closestream.h"
 #include "nls.h"
+#include "pathnames.h"
 #include "strutils.h"
 #include "xalloc.h"
 
@@ -88,47 +89,34 @@ enum {
 	OPT_JOURNALD,
 	OPT_RFC3164,
 	OPT_RFC5424,
-	OPT_SOCKET_ERRORS
+	OPT_SOCKET_ERRORS,
+	OPT_MSGID,
+	OPT_ID
 };
 
 struct logger_ctl {
 	int fd;
 	int pri;
 	pid_t pid;			/* zero when unwanted */
+	char *hdr;			/* the syslog header (based on protocol) */
 	char *tag;
-	char *unix_socket;
+	char *msgid;
+	char *unix_socket;		/* -u <path> or default to _PATH_DEVLOG */
 	char *server;
 	char *port;
 	int socket_type;
-	void (*syslogfp)(const struct logger_ctl *ctl, const char *msg);
+	size_t max_message_size;
+	void (*syslogfp)(struct logger_ctl *ctl);
 	unsigned int
 			unix_socket_errors:1,	/* whether to report or not errors */
 			prio_prefix:1,		/* read priority from intput */
 			stderr_printout:1,	/* output message to stderr */
 			rfc5424_time:1,		/* include time stamp */
 			rfc5424_tq:1,		/* include time quality markup */
-			rfc5424_host:1;		/* include hostname */
+			rfc5424_host:1,		/* include hostname */
+			skip_empty_lines:1; /* do not send empty lines when processing files */
 };
 
-static char *get_prio_prefix(char *msg, int *prio)
-{
-	int p;
-	char *end = NULL;
-	int facility = *prio & LOG_FACMASK;
-
-	errno = 0;
-	p = strtoul(msg + 1, &end, 10);
-
-	if (errno || !end || end == msg + 1 || end[0] != '>')
-		return msg;
-
-	if (p & LOG_FACMASK)
-		facility = p & LOG_FACMASK;
-
-	*prio = facility | (p & LOG_PRIMASK);
-	return end + 1;
-}
-
 static int decode(const char *name, CODE *codetab)
 {
 	register CODE *c;
@@ -171,6 +159,8 @@ static int pencode(char *s)
 	level = decode(s, prioritynames);
 	if (level < 0)
 		errx(EXIT_FAILURE, _("unknown priority name: %s"), s);
+	if(facility == LOG_KERN)
+		facility = LOG_USER; /* kern is forbidden */
 	return ((level & LOG_PRIMASK) | (facility & LOG_FACMASK));
 }
 
@@ -304,11 +294,58 @@ static char *xgetlogin(void)
 	return cp;
 }
 
-static void syslog_rfc3164(const struct logger_ctl *ctl, const char *msg)
+
+/* this creates a timestamp based on current time according to the
+ * fine rules of RFC3164, most importantly it ensures in a portable
+ * way that the month day is correctly written (with a SP instead
+ * of a leading 0). The function uses a static buffer which is
+ * overwritten on the next call (just like ctime() does).
+ */
+static const char *
+rfc3164_current_time(void)
 {
-	char *buf, pid[30], *cp, *tp, *hostname, *dot;
-	time_t now;
-	int len;
+	static char time[32];
+	struct timeval tv;
+	struct tm *tm;
+	static char *monthnames[] = { "Jan", "Feb", "Mar", "Apr",
+			"May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
+
+	gettimeofday(&tv, NULL);
+	tm = localtime(&tv.tv_sec);
+	snprintf(time, sizeof(time),"%s %2d %2.2d:%2.2d:%2.2d",
+		monthnames[tm->tm_mon], tm->tm_mday,
+		tm->tm_hour, tm->tm_min, tm->tm_sec);
+	return time;
+}
+
+/* writes generated buffer to desired destination. For TCP syslog,
+ * we use RFC6587 octet-stuffing. This is not great, but doing
+ * full blown RFC5425 (TLS) looks like it is too much for the
+ * logger utility.
+ */
+static void write_output(const struct logger_ctl *ctl, const char *const msg)
+{
+	char *buf;
+	const size_t len = xasprintf(&buf, "%s%s", ctl->hdr, msg);
+	if (write_all(ctl->fd, buf, len) < 0)
+		warn(_("write failed"));
+	else
+		if (ctl->socket_type == TYPE_TCP)
+			/* using an additional write seems like the best compromise:
+			 * - writev() is not yet supported by framework
+			 * - adding the \n to the buffer in formatters violates layers
+			 * - adding \n after the fact requires memory copy
+			 * - logger is not a high-performance app
+			 */
+			if (write_all(ctl->fd, "\n", 1) < 0)
+				warn(_("write failed"));
+	if (ctl->stderr_printout)
+		fprintf(stderr, "%s\n", buf);
+}
+
+static void syslog_rfc3164_header(struct logger_ctl *const ctl)
+{
+	char pid[30], *hostname, *dot;
 
 	*pid = '\0';
 	if (ctl->fd < 0)
@@ -316,55 +353,63 @@ static void syslog_rfc3164(const struct logger_ctl *ctl, const char *msg)
 	if (ctl->pid)
 		snprintf(pid, sizeof(pid), "[%d]", ctl->pid);
 
-	cp = ctl->tag ? ctl->tag : xgetlogin();
-
 	hostname = xgethostname();
 	dot = strchr(hostname, '.');
 	if (dot)
 		*dot = '\0';
 
-	time(&now);
-	tp = ctime(&now) + 4;
-
-	len = xasprintf(&buf, "<%d>%.15s %s %.200s%s: %.400s",
-		 ctl->pri, tp, hostname, cp, pid, msg);
-
-	if (write_all(ctl->fd, buf, len) < 0)
-		warn(_("write failed"));
-	if (ctl->stderr_printout)
-		fprintf(stderr, "%s\n", buf);
+	xasprintf(&ctl->hdr, "<%d>%.15s %s %.200s%s: ",
+		 ctl->pri, rfc3164_current_time(), hostname, ctl->tag, pid);
 
 	free(hostname);
-	free(buf);
 }
 
-static void syslog_rfc5424(const  struct logger_ctl *ctl, const char *msg)
+/* Some field mappings may be controversal, thus I give the reason
+ * why this specific mapping was used:
+ * APP-NAME <-- tag
+ *    Some may argue that "logger" is a better fit, but we think
+ *    this is better inline of what other implementations do. In
+ *    rsyslog, for example, the TAG value is populated from APP-NAME.
+ * PROCID <-- pid
+ *    This is a relatively straightforward interpretation from
+ *    RFC5424, sect. 6.2.6.
+ * MSGID <-- msgid (from --msgid)
+ *    One may argue that the string "logger" would be better suited
+ *    here so that a receiver can identify the sender process.
+ *    However, this does not sound like a good match to RFC5424,
+ *    sect. 6.2.7.
+ * Note that appendix A.1 of RFC5424 does not provide clear guidance
+ * of how these fields should be used. This is the case because the
+ * IETF working group couldn't arrive at a clear agreement when we
+ * specified RFC5424. The rest of the field mappings should be
+ * pretty clear from RFC5424. -- Rainer Gerhards, 2015-03-10
+ */
+#define NILVALUE "-"
+static void syslog_rfc5424_header(struct logger_ctl *const ctl)
 {
-	char *buf, *tag = NULL, *hostname = NULL;
-	char pid[32], time[64], timeq[80];
-#ifdef HAVE_SYS_TIMEX_H
-	struct ntptimeval ntptv;
-#endif
-	struct timeval tv;
-	struct tm *tm;
-	int len;
-
-	*pid = *time = *timeq = '\0';
 	if (ctl->fd < 0)
 		return;
 
+	char *time;
 	if (ctl->rfc5424_time) {
+		struct timeval tv;
+		struct tm *tm;
 		gettimeofday(&tv, NULL);
 		if ((tm = localtime(&tv.tv_sec)) != NULL) {
 			char fmt[64];
-
-			strftime(fmt, sizeof(fmt), " %Y-%m-%dT%H:%M:%S.%%06u%z",
-				 tm);
-			snprintf(time, sizeof(time), fmt, tv.tv_usec);
+			const size_t i = strftime(fmt, sizeof(fmt),
+					"%Y-%m-%dT%H:%M:%S.%%06u%z ", tm);
+			/* patch TZ info to comply with RFC3339 (we left SP at end) */
+			fmt[i-1] = fmt[i-2];
+			fmt[i-2] = fmt[i-3];
+			fmt[i-3] = ':';
+			xasprintf(&time, fmt, tv.tv_usec);
 		} else
 			err(EXIT_FAILURE, _("localtime() failed"));
-	}
+	} else
+		time = xstrdup(NILVALUE);
 
+	char *hostname;
 	if (ctl->rfc5424_host) {
 		hostname = xgethostname();
 		/* Arbitrary looking 'if (var < strlen()) checks originate from
@@ -372,41 +417,50 @@ static void syslog_rfc5424(const  struct logger_ctl *ctl, const char *msg)
 		if (255 < strlen(hostname))
 			errx(EXIT_FAILURE, _("hostname '%s' is too long"),
 			     hostname);
-	}
+	} else
+		hostname = xstrdup(NILVALUE);
 
-	tag = ctl->tag ? ctl->tag : xgetlogin();
-
-	if (48 < strlen(tag))
-		errx(EXIT_FAILURE, _("tag '%s' is too long"), tag);
+	char *const app_name = ctl->tag;
+	if (48 < strlen(ctl->tag))
+		errx(EXIT_FAILURE, _("tag '%s' is too long"), ctl->tag);
 
+	char *procid;
 	if (ctl->pid)
-		snprintf(pid, sizeof(pid), " %d", ctl->pid);
+		xasprintf(&procid, "%d", ctl->pid);
+	else
+		procid = xstrdup(NILVALUE);
 
+	char *const msgid = xstrdup((ctl->msgid) ? ctl->msgid : NILVALUE);
+
+	char *structured_data;
 	if (ctl->rfc5424_tq) {
-#ifdef HAVE_SYS_TIMEX_H
+#ifdef HAVE_NTP_GETTIME
+		struct ntptimeval ntptv;
 		if (ntp_gettime(&ntptv) == TIME_OK)
-			snprintf(timeq, sizeof(timeq),
-				 " [timeQuality tzKnown=\"1\" isSynced=\"1\" syncAccuracy=\"%ld\"]",
+			xasprintf(&structured_data,
+				 "[timeQuality tzKnown=\"1\" isSynced=\"1\" syncAccuracy=\"%ld\"]",
 				 ntptv.maxerror);
 		else
 #endif
-			snprintf(timeq, sizeof(timeq),
-				 " [timeQuality tzKnown=\"1\" isSynced=\"0\"]");
+			xasprintf(&structured_data,
+				 "[timeQuality tzKnown=\"1\" isSynced=\"0\"]");
 	}
 
-	len = xasprintf(&buf, "<%d>1%s%s%s %s -%s%s %s", ctl->pri, time,
-		  hostname ? " " : "",
-		  hostname ? hostname : "",
-		  tag, pid, timeq, msg);
-
-	if (write_all(ctl->fd, buf, len) < 0)
-		warn(_("write failed"));
-
-	if (ctl->stderr_printout)
-		fprintf(stderr, "%s\n", buf);
+	xasprintf(&ctl->hdr, "<%d>1 %s %s %s %s %s %s ",
+		ctl->pri,
+		time,
+		hostname,
+		app_name,
+		procid,
+		msgid,
+		structured_data);
 
+	free(time);
 	free(hostname);
-	free(buf);
+	/* app_name points to ctl->tag, do NOT free! */
+	free(procid);
+	free(msgid);
+	free(structured_data);
 }
 
 static void parse_rfc5424_flags(struct logger_ctl *ctl, char *optarg)
@@ -440,31 +494,23 @@ static int parse_unix_socket_errors_flags(char *optarg)
 	return AF_UNIX_ERRORS_AUTO;
 }
 
-static void syslog_local(const struct logger_ctl *ctl, const char *msg)
+static void syslog_local_header(struct logger_ctl *const ctl)
 {
-	char *buf, *tag;
-	char time[32], pid[32];
-	struct timeval tv;
-	struct tm *tm;
-	int len;
-
-	gettimeofday(&tv, NULL);
-	tm = localtime(&tv.tv_sec);
-	strftime(time, sizeof(time), "%h %e %T", tm);
-
-	tag = ctl->tag ? ctl->tag : program_invocation_short_name;
+	char pid[32];
 
 	if (ctl->pid)
 		snprintf(pid, sizeof(pid), "[%d]", ctl->pid);
 	else
 		pid[0] = '\0';
 
-	len = xasprintf(&buf, "<%d>%s %s%s: %s", ctl->pri, time, tag, pid, msg);
-	if (write_all(ctl->fd, buf, len) < 0)
-		warn(_("write failed"));
-	if (ctl->stderr_printout)
-		fprintf(stderr, "%s\n", buf);
-	free(buf);
+	xasprintf(&ctl->hdr, "<%d>%s %s%s: ", ctl->pri, rfc3164_current_time(),
+		ctl->tag, pid);
+}
+
+static void generate_syslog_header(struct logger_ctl *const ctl)
+{
+	free(ctl->hdr);
+	ctl->syslogfp(ctl);
 }
 
 static void logger_open(struct logger_ctl *ctl)
@@ -472,34 +518,41 @@ static void logger_open(struct logger_ctl *ctl)
 	if (ctl->server) {
 		ctl->fd = inet_socket(ctl->server, ctl->port, ctl->socket_type);
 		if (!ctl->syslogfp)
-			ctl->syslogfp = syslog_rfc5424;
+			ctl->syslogfp = syslog_rfc5424_header;
 		return;
 	}
-	if (ctl->unix_socket) {
-		ctl->fd = unix_socket(ctl, ctl->unix_socket, ctl->socket_type);
-		if (!ctl->syslogfp)
-			ctl->syslogfp = syslog_rfc5424;
-		return;
-	}
-	ctl->fd = unix_socket(ctl, "/dev/log", ctl->socket_type);
-	ctl->syslogfp = syslog_local;
+	if (!ctl->unix_socket)
+		ctl->unix_socket = _PATH_DEVLOG;
+
+	ctl->fd = unix_socket(ctl, ctl->unix_socket, ctl->socket_type);
+	if (!ctl->syslogfp)
+		ctl->syslogfp = syslog_local_header;
+	if(!ctl->tag)
+			ctl->tag = xgetlogin();
+	generate_syslog_header(ctl);
 }
 
 static void logger_command_line(const struct logger_ctl *ctl, char **argv)
 {
-	char buf[4096];
+	/* note: we never re-generate the syslog header here, even if we
+	 * generate multiple messages. If so, we think it is the right thing
+	 * to do to report them with the same timestamp, as the user actually
+	 * intended to send a single message.
+	 */
+	char *const buf = xmalloc(ctl->max_message_size + 1);
 	char *p = buf;
-	const char *endp = buf + sizeof(buf) - 2;
+	const char *endp = buf + ctl->max_message_size - 1;
 	size_t len;
 
 	while (*argv) {
 		len = strlen(*argv);
 		if (endp < p + len && p != buf) {
-			ctl->syslogfp(ctl, buf);
+			write_output(ctl, buf);
 			p = buf;
 		}
-		if (sizeof(buf) - 1 < len) {
-			ctl->syslogfp(ctl, *argv++);
+		if (ctl->max_message_size < len) {
+			(*argv)[ctl->max_message_size] = '\0'; /* truncate */
+			write_output(ctl, *argv++);
 			continue;
 		}
 		if (p != buf)
@@ -508,27 +561,61 @@ static void logger_command_line(const struct logger_ctl *ctl, char **argv)
 		*(p += len) = '\0';
 	}
 	if (p != buf)
-		ctl->syslogfp(ctl, buf);
+		write_output(ctl, buf);
 }
 
 static void logger_stdin(struct logger_ctl *ctl)
 {
-	char *msg;
 	int default_priority = ctl->pri;
-	char buf[1024];
+	int last_pri = default_priority;
+	size_t max_usrmsg_size = ctl->max_message_size - strlen(ctl->hdr);
+	char *const buf = xmalloc(max_usrmsg_size + 2 + 2);
+	int pri;
+	int c;
+	size_t i;
 
-	while (fgets(buf, sizeof(buf), stdin) != NULL) {
-		int len = strlen(buf);
+	c = getchar();
+	while (c != EOF) {
+		i = 0;
+		if (ctl->prio_prefix) {
+			if (c == '<') {
+				pri = 0;
+				buf[i++] = c;
+				while(isdigit(c = getchar()) && pri <= 191) {
+						buf[i++] = c;
+						pri = pri * 10 + c - '0';
+				}
+				if (c != EOF && c != '\n')
+					buf[i++] = c;
+				if (c == '>' && 0 <= pri && pri <= 191) { /* valid RFC PRI values */
+					i = 0;
+					if (pri < 8)
+						pri |= 8; /* kern facility is forbidden */
+					ctl->pri = pri;
+				} else
+					ctl->pri = default_priority;
 
-		/* some glibc versions are buggy, they add an additional
-		 * newline which is removed here.  */
-		if (0 < len && buf[len - 1] == '\n')
-			buf[len - 1] = '\0';
-		msg = buf;
-		ctl->pri = default_priority;
-		if (ctl->prio_prefix && msg[0] == '<')
-			msg = get_prio_prefix(msg, &ctl->pri);
-		ctl->syslogfp(ctl, msg);
+				if (ctl->pri != last_pri) {
+					generate_syslog_header(ctl);
+					max_usrmsg_size = ctl->max_message_size - strlen(ctl->hdr);
+					last_pri = ctl->pri;
+				}
+				if (c != EOF && c != '\n')
+					c = getchar();
+			}
+		}
+
+		while (c != EOF && c != '\n' && i < max_usrmsg_size) {
+			buf[i++] = c;
+			c = getchar();
+		}
+		buf[i] = '\0';
+
+		if(i > 0 || !ctl->skip_empty_lines)
+			write_output(ctl, buf);
+
+		if (c == '\n') /* discard line terminator */
+			c = getchar();
 	}
 }
 
@@ -536,6 +623,7 @@ static void logger_close(const struct logger_ctl *ctl)
 {
 	if (close(ctl->fd) != 0)
 		err(EXIT_FAILURE, _("close failed"));
+	free(ctl->hdr);
 }
 
 static void __attribute__ ((__noreturn__)) usage(FILE *out)
@@ -547,19 +635,23 @@ static void __attribute__ ((__noreturn__)) usage(FILE *out)
 	fputs(_("Enter messages into the system log.\n"), out);
 
 	fputs(USAGE_OPTIONS, out);
-	fputs(_(" -i, --id[=<id>]          log <id> (default is PID)\n"), out);
+	fputs(_(" -i                       log the logger command's PID\n"), out);
+	fputs(_("     --id[=<id>]          log the given <id>, or otherwise the PID\n"), out);
 	fputs(_(" -f, --file <file>        log the contents of this file\n"), out);
+	fputs(_(" -e, --skip-empty         do not log empty lines when processing files\n"), out);
 	fputs(_(" -p, --priority <prio>    mark given message with this priority\n"), out);
 	fputs(_("     --prio-prefix        look for a prefix on every line read from stdin\n"), out);
 	fputs(_(" -s, --stderr             output message to standard error as well\n"), out);
+	fputs(_(" -S, --size <size>        maximum size for a single message\n"), out);
 	fputs(_(" -t, --tag <tag>          mark every line with this tag\n"), out);
 	fputs(_(" -n, --server <name>      write to this remote syslog server\n"), out);
 	fputs(_(" -P, --port <number>      use this UDP port\n"), out);
 	fputs(_(" -T, --tcp                use TCP only\n"), out);
 	fputs(_(" -d, --udp                use UDP only\n"), out);
 	fputs(_("     --rfc3164            use the obsolete BSD syslog protocol\n"), out);
-	fputs(_("     --rfc5424[=<snip>]   use the syslog protocol (the default);\n"
+	fputs(_("     --rfc5424[=<snip>]   use the syslog protocol (the default for remote);\n"
 		"                            <snip> can be notime, or notq, and/or nohost\n"), out);
+	fputs(_("     --msgid              set rfc5424 MSGID field, ignored for non-rfc5424 format\n"), out);
 	fputs(_(" -u, --socket <socket>    write to this Unix socket\n"), out);
 	fputs(_("     --socket-errors[=<on|off|auto>]\n"
 		"                          print connection errors when using Unix sockets\n"), out);
@@ -586,17 +678,21 @@ int main(int argc, char **argv)
 	struct logger_ctl ctl = {
 		.fd = -1,
 		.pid = 0,
-		.pri = LOG_NOTICE,
+		.pri = LOG_USER | LOG_NOTICE,
 		.prio_prefix = 0,
 		.tag = NULL,
 		.unix_socket = NULL,
 		.unix_socket_errors = 0,
 		.server = NULL,
 		.port = NULL,
+		.hdr = NULL,
+		.msgid = NULL,
 		.socket_type = ALL_TYPES,
+		.max_message_size = 1024,
 		.rfc5424_time = 1,
 		.rfc5424_tq = 1,
 		.rfc5424_host = 1,
+		.skip_empty_lines = 0
 	};
 	int ch;
 	int stdout_reopened = 0;
@@ -605,7 +701,7 @@ int main(int argc, char **argv)
 	FILE *jfd = NULL;
 #endif
 	static const struct option longopts[] = {
-		{ "id",		optional_argument,  0, 'i' },
+		{ "id",		optional_argument,  0, OPT_ID },
 		{ "stderr",	no_argument,	    0, 's' },
 		{ "file",	required_argument,  0, 'f' },
 		{ "priority",	required_argument,  0, 'p' },
@@ -621,6 +717,9 @@ int main(int argc, char **argv)
 		{ "prio-prefix", no_argument, 0, OPT_PRIO_PREFIX },
 		{ "rfc3164",	no_argument,  0, OPT_RFC3164 },
 		{ "rfc5424",	optional_argument,  0, OPT_RFC5424 },
+		{ "size",       required_argument,  0, 'S' },
+		{ "msgid",      required_argument,  0, OPT_MSGID },
+		{ "skip-empty", no_argument,  0, 'e' },
 #ifdef HAVE_LIBSYSTEMD
 		{ "journald",   optional_argument,  0, OPT_JOURNALD },
 #endif
@@ -632,7 +731,7 @@ int main(int argc, char **argv)
 	textdomain(PACKAGE);
 	atexit(close_stdout);
 
-	while ((ch = getopt_long(argc, argv, "f:i::p:st:u:dTn:P:Vh",
+	while ((ch = getopt_long(argc, argv, "ef:ip:S:st:u:dTn:P:Vh",
 					    longopts, NULL)) != -1) {
 		switch (ch) {
 		case 'f':		/* file to log */
@@ -640,7 +739,13 @@ int main(int argc, char **argv)
 				err(EXIT_FAILURE, _("file %s"), optarg);
 			stdout_reopened = 1;
 			break;
+		case 'e':
+			ctl.skip_empty_lines = 1;
+			break;
 		case 'i':		/* log process id also */
+			ctl.pid = getpid();
+			break;
+		case OPT_ID:
 			if (optarg) {
 				const char *p = optarg;
 
@@ -662,6 +767,10 @@ int main(int argc, char **argv)
 		case 'u':		/* unix socket */
 			ctl.unix_socket = optarg;
 			break;
+		case 'S':		/* max message size */
+			ctl.max_message_size = strtosize_or_err(optarg,
+				_("failed to parse message size"));
+			break;
 		case 'd':
 			ctl.socket_type = TYPE_UDP;
 			break;
@@ -683,13 +792,18 @@ int main(int argc, char **argv)
 			ctl.prio_prefix = 1;
 			break;
 		case OPT_RFC3164:
-			ctl.syslogfp = syslog_rfc3164;
+			ctl.syslogfp = syslog_rfc3164_header;
 			break;
 		case OPT_RFC5424:
-			ctl.syslogfp = syslog_rfc5424;
+			ctl.syslogfp = syslog_rfc5424_header;
 			if (optarg)
 				parse_rfc5424_flags(&ctl, optarg);
 			break;
+		case OPT_MSGID:
+			if(strchr(optarg, ' '))
+					err(EXIT_FAILURE, _("--msgid cannot contain space"));
+			ctl.msgid = optarg;
+			break;
 #ifdef HAVE_LIBSYSTEMD
 		case OPT_JOURNALD:
 			if (optarg) {
